/* classicon.c - GNOME WM interaction helper functions
 * Copyright (C) 1999 Sam Solon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include "classicon.h"


/*
 * Helper function to actually look for the desktop file.
 */

static gchar*
SearchDesktopFile (gchar* startDir, gchar** theFileNames)
{
  struct dirent *dent;
  DIR   *chkdir;
  gchar **chkname = NULL;
  gchar *retname = NULL;
  
  /*
   * Loop over every entry looking for a match, recurse on directories
   */

  chkdir = opendir (startDir);
  g_return_val_if_fail (chkdir != NULL, NULL);

  while ((dent = readdir (chkdir))) {
    for (chkname = theFileNames; chkname && *chkname; chkname++) {
      if (strcmp (*chkname, dent->d_name) == 0) {
	retname = g_concat_dir_and_file (startDir, *chkname);
	break;
      }
    }

    if (retname != NULL)
      break;
  }

  /* Go through again and this time check subdirectories */

  if (retname == NULL) {
    rewinddir (chkdir);
    
    while ((dent = readdir (chkdir))) {
      struct stat chkstat;
      /* Ignore up and self links */

      if ((strcmp (dent->d_name, ".") != 0)
	  &&
	  (strcmp (dent->d_name, "..") != 0)) {
	gchar* chkpath = g_concat_dir_and_file (startDir, dent->d_name);

	if (stat (chkpath, &chkstat) == 0) { /* ignore errors */
	  if (S_ISDIR (chkstat.st_mode)) {
	    retname = SearchDesktopFile (chkpath, theFileNames);
	  }
	}

	g_free (chkpath);
	if (retname != NULL)
	  break;
      }
    }
  }

  closedir (chkdir);

  return retname;
}
    
    
/**
 * FindDesktopFile
 *
 * @Appnames: NULL terminated list of names
 *
 * @return: g_alloc'd string with fully qualified name of desktop file.
 *
 * This routine looks through the gnome/apps section of datadir for a file
 * named Appname.desktop and returns the path.
 */

static gchar*
FindDesktopFile (gchar **theAppNames)
{
  gchar *startdir = gnome_datadir_file ("gnome/apps");
  gchar *result = NULL;
  gchar **filenames = NULL;
  gint numnames = 0;
  gint i;
  
  g_return_val_if_fail (startdir != NULL, NULL);
  g_return_val_if_fail (theAppNames != NULL, NULL);
  
  /* Turn the app names into file names */

  for (numnames = 0; theAppNames[numnames]; numnames++)
    ;

  filenames = g_new0(gchar*, numnames+1);

  for (i = 0; i < numnames; i++)
    filenames[i] = g_strconcat (theAppNames[i], ".desktop", NULL);
  
  result = SearchDesktopFile (startdir, filenames);

  g_free (startdir);
  for (i = 0; i < numnames; i++)
    g_free (filenames[i]);

  g_free (filenames);
  
  return result;
}

/**
 * icon_for_class
 *
 * @className: string with comma separated class names
 * @iconWidth: width for returned icon
 * @iconHeight:height for returned icon
 * @retPixmap: returned pixmap
 * @retMask:   returned mask
 *
 * @return: TRUE if an icon was found
 *
 * Use the class(es) passed to find an icon and return it at the size
 * requested.
 */

gboolean
icon_for_class (gchar *className, gint iconWidth, gint iconHeight,
	      GdkPixmap **retPixmap, GdkBitmap **retMask)
{
  gboolean res = FALSE;
  gchar **classnames = g_strsplit (className, ",", 64);
  gchar* dfname = FindDesktopFile (classnames);

  if (dfname) {

    /* Right now config path is absolute but this should be handled by sub */

    gchar *path = g_strdup_printf ("=%s=/Desktop Entry/Icon", dfname);
    gchar *iconname = gnome_config_get_string (path);
    
    gchar *iconpath= gnome_pixmap_file (iconname);
    
    if (path) {
      GtkWidget *pixmap;

      pixmap = gnome_pixmap_new_from_file_at_size (iconpath,
						   iconWidth,
						   iconHeight);
      if (pixmap) {
	*retPixmap = GNOME_PIXMAP (pixmap)->pixmap;
	*retMask   = GNOME_PIXMAP (pixmap)->mask;
	gdk_pixmap_ref (*retPixmap);
	gdk_pixmap_ref (*retMask);

	gtk_widget_destroy (pixmap);

	res = TRUE;
      }
    }

    g_free (path);
    g_free (iconname);
    g_free (iconpath);
  }
  
  g_free (dfname);
  g_strfreev (classnames);

  return res;
}

      
